/*
 * SUGGEST INTERFACE INCLUDE V2.00 FEB 96
 *
 * Modifications by Thierry Godefroy:
 *
 *  24/03/99    Added typedefs.
 */


#ifndef _SUGGESTUSER_H

#ifndef _SYS_TYPES_H
#include <sys/types.h>
#endif

#ifndef _THINGS_H
#include <things.h>
#endif

#define _SUGGESTUSER_H


/* local QL STRING structure
   size of s varies, terminated in null
*/
typedef struct LQLSTR {
        short    l;
        char     s[2];
        } LQLSTR_t;

typedef short DISTTYP;

/* suggestion words are held in this structure
   read only
*/
typedef struct WORD {
        DISTTYP     dist, score;
        LQLSTR_t    s;
        } WORD_t;


/* array of sorted pointers to words
   read only
*/
typedef struct ARRAY {
        WORD_t  **base;
        int     size;
        } ARRAY_t;

/* max dictionary word length */
#define         MAXWORD         128

typedef enum    _ALLTYPE {NONE=0, EXPLICIT=1, IMPLICIT=2} ALLTYPE;

typedef struct _ALLREC ALLREC;
typedef struct _ALL    ALL;

/* public section of globals structure only */
typedef struct GLOBALS {
        void     *spare;
        short    root_cut;
        ARRAY_t  wordarray;
        char     word[MAXWORD];
        chanid_t spell;
        ALLTYPE  alltype;
        struct   ALLBUF   {
                 short  l;
                 char   s[MAXWORD];
                 } allbuf;
        ALLREC   *allrec;
        } GLOBALS_t;

/* refere to n th most probable word in list as QDOS string */
#define WORD_REF(globals, n)          (globals->wordarray.base[n])

/* a QDOS string */
#define QDOSSTR_REF(globals, n)       (WORD_REF(globals,n)->s)

/* and as C string */
#define CSTRING_REF(globals, n)       (QDOSSTR_REF(globals,n).s)

/* suggest info structure */
typedef struct SUG_INFO {
        char    default_rules[58];
        long    min_stack;
        short   typo_mode;
        short   minword;
        short   explicit_all_only;
        long    default_allsize;
        char    spare[200];
        } SUG_INFO_t;


/* suggest thing header -
   executable thing +
   array of entry point vectors
*/
typedef struct SUGGEST_HEAD {
        THING_HEADER_EXEC   head;
        long        (*alloc_global_P)(GLOBALS_t **globals, chanid_t spellch,
                                      LQLSTR_t *jobname, chanid_t owner_con,
                                      chanid_t owner, jobid_t spell_owner);
        long        (*stacklimit_P)(GLOBALS_t *globals, char *limit);
        long        (*infile_P)(GLOBALS_t *globals, char *name, short addrules);
        long        (*killfile_P)(GLOBALS_t *globals);
        long        (*findwords_P)(GLOBALS_t *globals, short newword);
        long        (*freemem_P)(GLOBALS_t *globals);
        LQLSTR_t    *(*find_note_P)(WORD_t *wp);
        SUG_INFO_t  *(*suggest_info_P)(SUG_INFO_t *buf);
        long        (*private_all_P)(ALLREC **allrec);
        long        (*public_all_P)(ALLREC **allrec);
        long        (*clear_all_P)(ALLREC *allrec);
        long        (*save_all_P)(ALLREC *allrec, char *filename);
        long        (*load_all_P)(ALLREC *allrec, char *filename);
        long        (*resize_all_P)(ALLREC *allrec, size_t new_size);
        long        (*addall_P)(ALLREC *allrec, char *for_word,
                                char *use, ALLTYPE type);
        long        (*check_all_P)(ALLREC *allrec, struct ALLBUF *buffer,
                                   char *for_this, ALL **record);
        LQLSTR_t    *(*pull_dn_P)(GLOBALS_t *globals, LQLSTR_t *title);
        long        (*cmdpd_P)(GLOBALS_t *globals, void *wman);
        } SUGGEST_HEAD_t;

/* suggest extra error messages */

#define  NORULES        "No rules"
#define  NOWORD         "No word"
#define  STACKOVER      "Stack over flow"
#define  NOTMEMFILE     "Not a history file"
#define  SIZENOTPERM    "Size not permitted"

#endif
